; Copyright (C) 2014 Texas Instruments Incorporated - http://www.ti.com/
;
; Redistribution and use in source and binary forms, with or without
; modification, are permitted provided that the following conditions
; are met:
;
; Redistributions of source code must retain the above copyright
; notice, this list of conditions and the following disclaimer.
;
; notice, this list of conditions and the following disclaimer in the
; documentation and/or other materials provided with the
; distribution.
;
; Neither the name of Texas Instruments Incorporated nor the names of
; its contributors may be used to endorse or promote products derived
; from this software without specific prior written permission.
;
; THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
; "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
; LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
; A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
; OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
; SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
; LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
; DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
; THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
; (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
; OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
;
; 	file:   spi1.asm
;
; 	brief: spi communication between AM437x and ADC
;
;	Version		Description						Author
; 	0.1      SPI PRU Communicatin		    Karthik Bandi
;
	.sect	".text: 0x00000000"
	.retain 	; Required for building .out with assembly file
	.retainrefs ; Required for building .out with assembly file
	.sslist		; Required for building registers with names

	.global main

; connections between AM437x and ADC
; AM437x					ADC
; nCS(R30.t13)		to		CS
; SCLK(R30.t17)		to 		SCLK
; MOSI(R30.t16)		to 		SDI/MOSI
; MISO(R31.b0)		to 		SDO/MISO

SPI_CS		.set 13			; connected to 19 pin on AM J16 as GPO13
SPI_SCLK	.set 17			; connected to 23 pin on AM J16 as GPO17
SPI_MOSI	.set 16			; connected to 5 pin on AM J16 as GPO16
; MISO is connected to 3 pin on AM J16 as GPI0
toggle_output	.set 15			; connected to 21 pin on AM J16 as GPO15
; Assigning regiters with names
	.asg R5.w0, tx_data
	.asg R5.w2, rx_data
	.asg R6.b0, loop_count
	.asg R31.b0, input_reg
	.asg R11.b0, temp1
	.asg R2.b0, freq_sel
	.asg R4.b0, delay_bit
	.asg R4.b1, delaybit_actual
	.asg R4.b2, delaybit_measured
;Defining Macros
n_op .macro
	 add R0, R0, 0
	.endm

SPI_17MHZ .set 0
SPI_10MHZ .set 1
SPI_5MHZ .set 2
SPI_1MHZ .set 3

SPI_FREQUENCY_17MHZ .set SPI_17MHZ
SPI_FREQUENCY_10MHZ .set SPI_10MHZ
SPI_FREQUENCY_5MHZ .set SPI_5MHZ
SPI_FREQUENCY_1MHZ .set SPI_1MHZ

main:

		zero 	&R0, 120
		set		R30, R30, SPI_CS
		ldi 	R9, 0
		ldi		R28, 0
adc_reset:
		;resetting the device
		;ldi		tx_data, 0x8500
		;jal		R29.w0, fn_spi_transaction
adc_reset_done:

adc_init:
		; loading data into R5.w0 for writing to ADC
		;writing data to the program registers
		ldi		tx_data, 0x03A5
		jal		R29.w0, fn_spi_transaction
		;setting the ch_0 to operate at +-0.625*VREF(4.096V) voltage level
		ldi		tx_data, 0x0502
		jal		R29.w0, fn_spi_transaction
		; selecting the channel 0(ch_0)
		ldi		tx_data, 0xC000
		jal		R29.w0, fn_spi_transaction
start_timer:
        ; enable eCAP periodic timer event
        lbco	&R28.w0, c3, 0x2A, 2				;ICSS_ECAP, ICSS_eCAP_ECCTL2(2Ah=42)
        set		R28.w0, R28.w0, 4
		sbco	&R28.w0, c3, 0x2A, 2				;ICSS_ECAP, ICSS_eCAP_ECCTL2(2Ah=42)

adc_data_acquisition:
		;wait untill the event is occured
event_occured:
		lbco	&R28.w0, c3, 0x2E, 2				;ICSS_ECAP, ICSS_eCAP_ECFLG
		qbbc	event_occured, R28.w0, 6
		; clear ECAP event in ECAP module
		ldi		R28.w0, (1<<6 | 1<<0) 			;(1 << ECAP_CTR_PRD_SHIFT) | (1 << ECAP_INT_SHIFT) of ECCLR
		sbco	&R28.w0, c3, 0x30, 2				;ICSS_ECAP, ICSS_eCAP_ECCLR(30h=48)

		; continue in ch_0 to read the data
		ldi		tx_data, 0x0000
		jal		R29.w0, fn_spi_transaction
		sbco	&R5.w2, c28, R9.w0, 2			;storing the sampled data in to the PRU_Device_memory location 0x00050000
		;qbbs	toggle_data, R30, 15
		;qbbc	toggle_data, R30, 15
		;jmp		continue_data_acquisition
;toggle_data:
		;xor		R30, R30, 15
;continue_data_acquisition:
		add		R9.w0, R9.w0, 2
		and		R9.b1, R9.b1, 0x07
		qba		adc_data_acquisition

		ldi		tx_data, 0x0398
		jal		R29.w0, fn_spi_transaction

		ldi		tx_data, 0x0336
		jal		R29.w0, fn_spi_transaction

		ldi		tx_data, 0x0301
		jal		R29.w0, fn_spi_transaction
adc_init_done:

		qba		adc_data_acquisition
		;HALT
		; TX data (16-bit) in R5.w0
		; RX data (16-bit) in R5.w2
		; loop counter in R6.b0

;****************************************************************************
;
;     NAME                   : fn_spi_transaction
;     DESCRIPTION            : This function is used for the transmission and receving of data from AM437x and ADC
;
;     RETURNS                : R29.w0
;     ARGS                   : NA
;     USES                   : Registers
;     INVOKES                : NA
;
;****************************************************************************
fn_spi_transaction:
		ldi		loop_count, 0
		ldi 	R1.b0, &temp1		;load R11.b0 address to R1.b0
		ldi		R8, 0x00000000		;loading PRU_ICSS0_PRU1 data ram address
; for selection of clock

		lbbo	&freq_sel, R8, 0, 1
		lbbo	&R3, R8, 4, 4
		lbbo	&delay_bit, R8, 8, 1
		qble	clk_cycle_count, delay_bit, 7
		ldi		R2.b2, 32
		jmp		ncs_assert
clk_cycle_count:
		ldi		R2.b2, 33
ncs_assert:
		clr		R30, R30, SPI_CS	; assert CS
;****************************************************************************
;
;     NAME                   : fn_clk_selection
;     DESCRIPTION            : This function is used for clock selection by the user to run at different frequencies(17,10,5,1MHzs)
;
;     RETURNS                : init_frequency
;     ARGS                   : freq_sel
;     USES                   : R2.b0
;     INVOKES                : NA
;
;****************************************************************************
fn_clk_selection:
		qbeq	init_freq_17MHz, freq_sel, SPI_FREQUENCY_17MHZ
		qbeq	init_freq_10MHz, freq_sel, SPI_FREQUENCY_10MHZ
		qbeq	init_freq_5MHz, freq_sel, SPI_FREQUENCY_5MHZ
		qbeq	init_freq_1MHz, freq_sel, SPI_FREQUENCY_1MHZ
		HALT	; invalid frequency
init_freq_17MHz:
				;Transmission of data to ADC
spi_tx_loop:
		; set or clear the MOSI signal for data transmission
		qbbc 	spi_tx_dataclr, tx_data, 15
		set 	R30, R30, SPI_MOSI
		jmp 	spi_tx_datashift
spi_tx_dataclr:
		clr 	R30, R30, SPI_MOSI
		n_op
spi_tx_datashift:

		; ADC will sample with falling edge
		set 	R30, R30, SPI_SCLK
		lsl 	tx_data, tx_data, 1
		loop 	spi_tx_setclk, R3.b0		;loop for 8 times for 10MHz and 4 times for 17MHz
		n_op
spi_tx_setclk:
		clr		R30, R30, SPI_SCLK
spi_tx_clrclk:
		add		loop_count, loop_count, 1
		qbgt	spi_tx_loop, loop_count, 16
		; Receiving of Data from ADC.
		loop 	spi_tx_clk, 3
		n_op
spi_tx_clk:
End_spi_tx_loop:

spi_rx_loop:
		set 	R30, R30, SPI_SCLK
		loop 	spi_rx_setclk, R3.b2
		n_op
spi_rx_setclk:
		clr		R30, R30, SPI_SCLK
		mvib 	*R1.b0++, input_reg	;R1.b0 points to R11.b0 address and increments the R11.b0 address and moves R31.b0 to R11.bn
		loop 	spi_rx_clrclk, R3.b3
		n_op
spi_rx_clrclk:
		; code to analyze oversampled data and get the result 16-bits

		add		loop_count, loop_count, 1
		qbgt	spi_rx_loop, loop_count, R2.b2

		set		R30, R30, SPI_CS
		loop 	spi_cs_set, 4
		n_op
spi_cs_set:
		;here delay bit calculation
;****************************************************************************
;
;     NAME                   : delay_bit_measurement
;     DESCRIPTION            : This function is used for the delay bit calculation
;
;     RETURNS                : delaybit-measured value
;     ARGS                   : delaybit_measured, R11.b0
;     USES                   : R4.b3, R11.b0, R11.b1
;     INVOKES                : NA
;
;****************************************************************************
fn_delay_bit_measurement:
; Here I need to calculate the Delay for once and depending upon that we can use for sampling(optional)
		;ldi 	R1.b0, &R11.b0
		lmbd	delaybit_measured, R11.b0, 1
		qbne	sampling_data, delaybit_measured, 32
		lmbd	delaybit_measured, R11.b1, 1
		add		delaybit_measured, delaybit_measured, 8
sampling_data:
		qbgt	sampling_data_delay8_15, delay_bit, 15
		halt
sampling_data_delay8_15:
		ldi 	R1.b0, &R11.b1
		sub		delaybit_actual, delay_bit, 8
		loop	shift_data_done_delay8_15, 16
		mvib	R7.b0, *R1.b0++
		lsl 	rx_data, rx_data, 1
		qbbs	set_dataout_bit_delay8_15, R7.b0, delaybit_actual
		clr		rx_data, rx_data, 0		; clearing the data out bit depending upon input
		jmp		shift_data_done_delay8_15
set_dataout_bit_delay8_15:
		set		rx_data, rx_data, 0		; Setting the data out bit depending upon input
shift_data_done_delay8_15:
		jmp		R29.w0
End_spi_rx_loop:
end_feq_17MHz:

; same tx loop for the frequencies below
init_freq_10MHz:
init_freq_5MHz:
init_freq_1MHz:

spi_tx_loop1:
		; set or clear the MOSI signal for data transmission
		qbbc 	spi_tx_dataclr1, tx_data, 15
		set 	R30, R30, SPI_MOSI
		jmp 	spi_tx_datashift1
spi_tx_dataclr1:
		clr 	R30, R30, SPI_MOSI
		n_op
spi_tx_datashift1:
		lsl 	tx_data, tx_data, 1
		; ADC will sample with falling edge
		set 	R30, R30, SPI_SCLK
		loop 	spi_tx_setclk1, R3.b0
		n_op
spi_tx_setclk1:
		clr		R30, R30, SPI_SCLK
		loop 	spi_tx_clrclk1, R3.b1
		n_op
spi_tx_clrclk1:
		add		loop_count, loop_count, 1
		qbgt	spi_tx_loop1, loop_count, 16
		; Receiving of Data from ADC.
		loop 	spi_tx_clk1, 3
		n_op
spi_tx_clk1:
End_spi_tx_loop1:

spi_rx_loop1:
		set 	R30, R30, SPI_SCLK
		loop 	spi_rx_setclk1, R3.b2
		n_op
spi_rx_setclk1:
		clr		R30, R30, SPI_SCLK
		mvib 	*R1.b0++, input_reg	;R1.b0 points to R11.b0 address and increments the R11.b0 address and moves R31.b0 to R11.bn
		loop 	spi_rx_clrclk1, R3.b3
		n_op
spi_rx_clrclk1:
		; code to analyze oversampled data and get the result 16-bits

		add		loop_count, loop_count, 1
		qbgt	spi_rx_loop1, loop_count, R2.b2

		set		R30, R30, SPI_CS
		loop 	spi_cs_set1, 4
		n_op
spi_cs_set1:
		;here delay bit calculation
;****************************************************************************
;
;     NAME                   : delay_bit_measurement1
;     DESCRIPTION            : This function is used for measuring the delay bit
;
;     RETURNS                : delaybit-measured register
;     ARGS                   : delaybit_measured, R11.b0
;     USES                   : R4.b3, R11.b0, R11.b1
;     INVOKES                : NA
;
;****************************************************************************
fn_delay_bit_measurement1:
; Here I need to calculate the Delay for once and depending upon that we can use for sampling(optional)
		;ldi 	R1.b0, &R11.b0
		lmbd	delaybit_measured, R11.b0, 1
		qbne	sampling_data1, delaybit_measured, 32
		lmbd	delaybit_measured, R11.b1, 1
		add		delaybit_measured, delaybit_measured, 8
sampling_data1:
		qbgt	sampling_data_delay5_6, delay_bit, 7
sampling_data_delay7_8:
		ldi 	R1.b0, &R11.b1
		sub		delaybit_actual, delay_bit, 5
		loop	shift_data_done_delay7_8, 16
		mvib	R7.b0, *R1.b0++
		lsl 	rx_data, rx_data, 1
		qbbs	set_dataout_bit_delay7_8, R7.b0, delaybit_actual
		clr		rx_data, rx_data, 0		; clearing the data out bit depending upon input
		jmp		shift_data_done_delay7_8
set_dataout_bit_delay7_8:
		set		rx_data, rx_data, 0		; Setting the data out bit depending upon input
shift_data_done_delay7_8:
		jmp		R29.w0
sampling_data_delay5_6:
		qbge	sampling_data_delay0_4, delay_bit, 4
		ldi 	R1.b0, &R11.b0
		sub		delaybit_actual, delay_bit, 2
		loop	shift_data_done_delay5_6, 16
		mvib	R7.b0, *R1.b0++
		lsl 	rx_data, rx_data, 1
		qbbs	set_dataout_bit_delay5_6, R7.b0, delaybit_actual
		clr		rx_data, rx_data, 0		; clearing the data out bit depending upon input
		jmp		shift_data_done_delay5_6
set_dataout_bit_delay5_6:
		set		rx_data, rx_data, 0		; Setting the data out bit depending upon input
shift_data_done_delay5_6:
		jmp		R29.w0
sampling_data_delay0_4:
		ldi 	R1.b0, &R11.b0
		sub		delaybit_actual, delay_bit, 1
		loop	shift_data_done_delay0_4, 16
		mvib	R7.b0, *R1.b0++
		lsl 	rx_data, rx_data, 1
		qbbs	set_dataout_bit_delay0_4, R7.b0, delaybit_actual
		clr		rx_data, rx_data, 0		; clearing the data out bit depending upon input
		jmp		shift_data_done_delay0_4
set_dataout_bit_delay0_4:
		set		rx_data, rx_data, 0		; Setting the data out bit depending upon input
shift_data_done_delay0_4:
		jmp		R29.w0

End_spi_rx_loop1:
End_fn_spi_tx_rx:
